/*
   cc283 - Intelligent Problem Solving - Assignment 2
   
   Robert Stacey - rpstac
   
   cc283assignment2_buyer.pl
   
   This program implements a Buyer in the negotiation game, the strategy used
   is to determine which profit region the offer is in and then try to maximise 
   the profit gained.

   The buyer also includes several clauses that try to stop the seller taking advantage
*/

% Below are the parameter that the buyer uses, they are the profit margins that
% it uses to calculate when to accept offer and what bids to make.

'buyer: parameter'(acceptable, 0.2).
'buyer: parameter'(prefered, 0.35).
'buyer: parameter'(liked, 0.5).
'buyer: parameter'(lastBid, 0.1).

/*
   The buyer predicate - buyer(Utility, DaysToBuy, BiddingHistory, Bid).

   The predicate takes the first three variables, the Utility and DaysToBuy as instantiated
   integers and the third Bidding History as a list in the form 
              [ +Int, -Int, +Int, ..... ]
   The predicate will instantiate the last variable Bid to an integer value.
*/

% This clause should stop the seller gaining extra money when the bid is already acceptable
buyer(_, _, [+Offer, -LastBid, +LastOffer|_], LastBid):-
	Offer > LastOffer,
	buyer_writeln(['-Buyer: The seller is trying to get more money so bid same again at ', LastBid]).


% This clause accepts an offer if on the last day to buy the price is below the 
% utility
buyer(Utility, 1, [+Offer|_], Offer):-
	Offer < Utility,
	buyer_writeln(['-Buyer: Last Day and price is ok, so accept at ', Offer]).

% This clause makes a bid within the lastbid profit region if the offer is
% not acceptable on the lastday
buyer(Utility, 1, [_, -LastBid|_], Bid):-
	bid(Utility, lastBid, Bid),
	Bid > LastBid,
	buyer_writeln(['-Buyer: Last day but price not good enough, bid in lastBid region at ', Bid]).

% This clause just repeats the last bid if the above bid would be less that the last bid
buyer(_, 1, [_, -LastBid|_], LastBid):-
	buyer_writeln(['-Buyer: Last day but bidding close to min profit so bid same again at ', LastBid]).

% This clause acccepts an offer that is within the prefered profit region with 
% 2 days left to buy.
buyer(Utility, 2, [+Offer|_], Offer):-
	bid(Utility, prefered, Profit),
	Offer < Profit,
	buyer_writeln(['-Buyer: 2 days to go, offer in prefered region so accept at ', Offer]).

% This clause makes a bid within the prefered profit region if the offer with 
% 2 days to sell is not acceptable.
buyer(Utility, 2, [_, -LastBid|_], Bid):-
	bid(Utility, prefered, Bid),
	Bid > LastBid,
	buyer_writeln(['-Buyer: 2 days to go, offer not acceptable so bid in prefered region at ', Bid]).

buyer(_, 2, [_, -LastBid|_], LastBid):-
	buyer_writeln(['-Buyer: 2 days to go, offer not acceptable so bid the same as close to utility']).

% These clauses should stop sellers that hold out at a certain price 
buyer(_, _, [+Offer, -Bid, +LastOffer, -LastBid, +BeforeLastOffer|_], Bid):-
	Offer =:= LastOffer,
	Offer =:= BeforeLastOffer,
	Bid < LastBid,
	buyer_writeln(['-Buyer: The seller is holding the offer so stay at ', Bid]).

buyer(_, _, [+Offer, _, +LastOffer, -Bid, +BeforeLastOffer|_], Bid):-
	Offer =:= LastOffer,
	Offer =:= BeforeLastOffer,
	buyer_writeln(['-Buyer: The seller is holding the offer so go back to earlier bid at ', Bid]).

% This clause takes the offer and decides whether to bid lower than
% it normally would because the offer is lower than it would bid
buyer(Utility, _, [+Offer], Bid):-
	ProposedBid is truncate(Utility*0.5),
	Offer < ProposedBid,
	Bid is truncate(Offer-Utility*0.2),
	buyer_writeln(['-Buyer: First offer lower than expected so bid lower than normal at ', Bid]).

% This clause makes the normal first bid
buyer(Utility, _, [_], Bid):-	
	Bid is truncate((Utility*0.4)),
	buyer_writeln(['-Buyer: First bid, so bid at 40% of Utility at ', Bid]).

% This clause looks to see if the offers have been high enough to accept
% for long enough, likedProfit region
buyer(Utility, _, [+Offer|History], Offer):- 
	below(Utility, [+Offer|History], liked, 2),
	buyer_writeln(['-Buyer: Offer has been in liked region for long enough to accept at ', Offer]).

% This clause bids in the liked profit region if the offer has only been
% in this region for 1 day.
buyer(Utility, _, [+Offer|History], Bid):-
	below(Utility, [+Offer|History], liked, 1),
	bid(Utility, liked, Profit),
	Offer is truncate(Offer - Utility*0.1),
	min(Offer, Profit, Bid),
	buyer_writeln(['-Buyer: Offer in liked region but just try to get more with ', Bid]).

% This clause looks to see if the offers have been high enough to accept
% for long enough, preferedProfit region
buyer(Utility, _, [+Offer|History], Offer):-
	below(Utility, [+Offer|History], prefered, 3),
	buyer_writeln(['-Buyer: Offer has been in prefered region for three days so accept at ', Offer]).

% This clause bids in the prefered profit region if the offer has been in 
% this region for two days or less
buyer(Utility, _, [+Offer|History], Bid):-
	below(Utility, [+Offer|History], prefered, 1),
	bid(Utility, prefered, Profit),
	Offer is truncate(Offer - (Utility*0.1)),
	min(Offer, Profit, Bid),
	buyer_writeln(['-Buyer: Offer has been in prefered region for <3 days, bid in region at ', Bid]).

% This clause looks to see if the offers have been high enough to accept
% for long enough, acceptableProfit region
buyer(Utility, _, [+Offer|History], Offer):-
	below(Utility, [+Offer|History], acceptable, 4),
	buyer_writeln(['-Buyer: Offer has been in acceptable region for four days so accept at ', Offer]).

% this clause bids in the prefered profit region if the offer has been in the 
% acceptable region for at least two days
buyer(Utility, _, [+Offer|History], Bid):-
	below(Utility, [+Offer|History], acceptable, 2),
	Offer is truncate(Offer - Utility*0.1),
	bid(Utility, prefered, Profit),
 	min(Offer, Profit, Bid),
	buyer_writeln(['-Buyer: Offer has been in acceptable region for >1 day, bid in prefered at ',Bid]).

% This clause bids in the liked profit region if the offer has been in the 
% acceptable region for just the one day
buyer(Utility, _, [+Offer|History], Bid):-
	below(Utility, [+Offer|History], acceptable, 1), 
	Offer is truncate(Offer - Utility*0.1),
	bid(Utility, liked, Profit),
	min(Offer, Profit, Bid),
	buyer_writeln(['-Buyer: Offer is in acceptable region, bid in liked at ', Bid]).

% This clause makes a bid if none of the above clauses work i.e. the offer 
% is not profitable enough
buyer(Utility, _, [_, -LastBid|_], Bid):-
	Bid is truncate(LastBid*1.075),
        Bid < Utility,
	buyer_writeln(['-Buyer: Offer not profitable so just increase lastbid to ', Bid]).

% This clause makes the bid if the offers are not profitable and the normal next offer would be 
% over the utility this just repeats the last bid.
buyer( _, _, [_, -Bid|_], Bid):-
	buyer_writeln(['-Buyer: Offer not profitable and limit reached just bid the same again at ', Bid]).



/* 
   Min predicate - min( X, Y, Z).
   
   This takes two instantiated arguements X and Y and returns the smallest in
   the third variable Z.
*/

% Return X as it is the smallest
min( X, Y, X):-
	X < Y.

% Return Y as it is the smallest or they are the same value
min( X, Y, Y):-
	Y =< X.


/*
   below predicate below(Utility, BiddingHistory, ProfitMargin, Days).

   The predicate takes the first three variables instantiated and then instantiates the 
   fourth to a value from 0 up to the number of offers. This value is the number of
   offers in a row that have been below a certain profit margin.
*/

% First offer above so return 1
below(Utility, [+Offer], Type, 1):-
	bid(Utility, Type, Profit),
	Offer < Profit.

% First deal with the other offers to see if they are below the margin, then see if this offer
% was below the margin.
below(Utility, [+Offer, _|History], Type, Days):-
	below(Utility, History, Type, DaysAbove),
	bid(Utility, Type, Profit),
	Offer < Profit,
	Days is DaysAbove+1.

% The offer is not below the margin so return 0
below(_, _, _, 0).



/*
   The bid predicate bid(Utility, ProfitMargin, Bid). 

   The predicate take the first two variables instantiated and it instantiates the third, Bid to
   a sutible value for the profit margin.
*/

% Return possible value for bid
bid(Utility, Type, Bid):-
	'buyer: parameter'(Type, Margin),
	Bid is truncate(Utility - Utility*Margin).

%=================================================================================================

/*
   The section below is taken from the code given by Edward Tsang so that the buyer outputs 
   reasoning in the same way as the other buyers.
*/

buyer_writeln(_) :- mediator_parameter(no_explanation), !.
	/* no output if mediator suppresses reasoning */
buyer_writeln([]) :- nl, !.
buyer_writeln([H|T]) :- write(H), buyer_writeln(T), !.


%=================================================================================================

