/*

jmcait_buyer.pl		by Jason Caits-Cheverst, March 2009.

A buyer script that's really no more than an elaborate if...elif...else statement,
which starts a discrete amount above the Utility specified and goes down until the
rival seller agrees.

The biggest assumption made in this is that the Util for me and the Cost for the
buyer aren't going to be more than a hundred or so apart. If there's a huge
distance between them, I imagine this script will fall down quite badly.

*/

'buyer:param'(minprof, 38).
'buyer:param'(desprof, 185).


/* FINAL ROUND */

/* If their price hasn't changed, accept it if it gives us profit. */
/* If their price gives us profit, accept it. */
/* If their price does not give us profit, don't accept it. */

buyer(Util, 1, [O|_], Nextprice) :-
	'buyer:param'(minprof, Mp),
	(Util - Mp) > O,
	Nextprice is O,
	buyer_writeln(['-Buyer--> FINAL ROUND--> ACCEPTING: ', Nextprice, '.']).

buyer(Util, 1, [O|_], Nextprice) :-
	'buyer:param'(minprof, Mp),
	O >= (Util - Mp),
	Nextprice is (Util - Mp),
	buyer_writeln(['-Buyer--> FINAL ROUND--> REJECTING: ', Nextprice, '.']).


/* BUY ME */

/* If their last offer is greater than our last offer, we accept. */

buyer(Util, _, [H,T|_], Nextprice) :-
	(0 - H) >= T,
	(0 - H) < Util,
	Nextprice is H,
	buyer_writeln(['-Buyer--> GOOD DEAL DETECTED--> BUYING: ', Nextprice, '.']).


/* DON'T BUY, I'M SNEAKY */

/* Underhanded Tactic 2: If the seller tries to regain any money, this should cap them. */

buyer(_, _, [O, S, F, _, _|_], S):-
	O > F,
	buyer_writeln(['-Buyer--> REGAIN ATTEMPT DETECTED--> FOILING: ', S]).


/* FIRST ROUND */

/* Offer our ideal price, mostly to test their curve. */

buyer(Util, _, [OSF], Nextprice) :-
	'buyer:param'(desprof, Dpr),
	Nextprice is (Util - Dpr),
	buyer_writeln(['-Buyer--> FIRST ROUND--> OFFERING: ', Nextprice, '.']).


/* SECOND ROUND */

/* We start on our curve. */

buyer(Util, DTB, [O,S,F], Nextprice) :-
	'buyer:param'(desprof, Dpr),
	Nextprice is (Util - Dpr),
	buyer_writeln(['-Buyer--> SECOND ROUND--> OFFERING: ', Nextprice, '.']).


/* NTH ROUND */

/* If the bid hasn't changed much, we accept it if it gives us a reasonable profit. */

buyer(Util, DTB, [O,S,F|T], Nextprice) :-
	'buyer:param'(desprof, Dpr),
	'buyer:param'(minprof, Mpr),
	(F - O) < 10,
	(Util - Mpr) >= O,
	buyer_writeln(['-Buyer--> NTH ROUND(1)--> OFFERING: ', O, '.']),
	Nextprice is O.

buyer(Util, DTB, [O,S,F|T], Nextprice) :-
	DTB < 3,
	'buyer:param'(desprof, Dpr),
	'buyer:param'(minprof, Mpr),
	(F - O) < 10,
	O > Util - Mpr,
	Nextprice is Util - Mpr,
	buyer_writeln(['-Buyer--> NTH ROUND(2)--> OFFERING: ', Nextprice, '.']).

buyer(Util, DTB, [O,S,F|T], Nextprice) :-
	'buyer:param'(desprof, Dpr),
	'buyer:param'(minprof, Mpr),
	Nextprice is (Util - (Dpr - (truncate((Dpr - Mpr) / DTB)))),
	buyer_writeln(['-Buyer--> NTH ROUND(3)--> OFFERING: ', Nextprice, '.']).


/* BUYER PRINT FUNCTIONS */

buyer_writeln(_) :- mediator_parameter(no_explanation), !.

buyer_writeln([]) :- !.

buyer_writeln([H|T]) :- write(H), buyer_writeln(T), !.