/*

jmcait_seller.pl	by Jason Caits-Cheverst, March 2009.

A seller script with faulty maths that tries to fit its bid acceptance policy
to a curve that's determined by our minimum profit margin but not by their curve.

I guess you can tell I put most of my effort into making my buyer work well.

*/


'seller:param'(minprof, 38).
'seller:param'(initmod, 3).


/* LAST ROUNDS */

/* Drop to minimum profit and do not yield. */

seller(Cost, 2, [PSF], AskingPrice) :-
	'seller:param'(minprof, Mpr),
	AskingPrice is (Cost + Mpr),
	seller_writeln(['-Seller: ROUND MAX-1: ', AskingPrice, '.']).

seller(Cost, 1, [P|SF], AskingPrice) :-
	'seller:param'(minprof, Mpr),
	AskingPrice is (Cost + Mpr),
	seller_writeln(['-Seller: ROUND MAX: ', AskingPrice, '.']).


/* SELL ME */

/* If their offer is higher than our last offer, accept. */

seller(_, _, [O,T|_], AskingPrice) :-
	(0 - O) > T,
	AskingPrice is (0 - O),
	seller_writeln(['-Seller: ROUND N: ', AskingPrice, '.']).


/* FIRST ROUND */

/* In which we make a fairly high starting offer, to give ourselves a beneficial curve. */

seller(Cost, DTS, [], AskingPrice) :-
	'seller:param'(initmod, Imod),
	AskingPrice is Cost * Imod,
	seller_writeln(['-Seller: ROUND 1: ', AskingPrice, '.']).


/* SECOND ROUND */

/* In which we see their reply. */

seller(Cost, DTS, [BuyAttempt, SellAttempt], AskingPrice) :-
	'seller:param'(initmod, Imod),
	'seller:param'(minprof, Mpr),
	BuyAttempt >= ((Cost + Mpr) * DTS),
	AskingPrice is BuyAttempt,
	seller_writeln(['-Seller: ROUND 2: PRICE GOOD: ', AskingPrice, '.']).

seller(Cost, DTS, [BuyAttempt, SellAttempt], AskingPrice) :-
	'seller:param'(initmod, Imod),
	'seller:param'(minprof, Mpr),
	AskingPrice is truncate((Cost + Mpr) * DTS),
	seller_writeln(['-Seller: ROUND 2: PRICE BAD: ', AskingPrice, '.']).

/* THIRD ROUND */

/* In which we try to anticipate where they'll end up. */

seller(Cost, DTS, [BuyAttempt, SellAttempt, BA2, SA2], AskingPrice) :-
	'seller:param'(initmod, Imod),
	'seller:param'(minprof, Mpr),
	(BuyAttempt - (BA2 - BuyAttempt)) >= (Cost + ((Cost * Imod) - (Cost + Mpr) / DTS)),
	AskingPrice is (BuyAttempt - (BA2 - BuyAttempt)),
	seller_writeln(['-Seller: ROUND 3: PRICE GOOD: ', AskingPrice, '.']).

seller(Cost, DTS, [BuyAttempt, SellAttempt, BA2, SA2], AskingPrice) :-
	'seller:param'(initmod, Imod),
	'seller:param'(minprof, Mpr),
	(BuyAttempt - (BA2 - BuyAttempt)) < (Cost + ((Cost * Imod) - (Cost + Mpr) / DTS)),
	AskingPrice is truncate(Cost + ((Cost * Imod) - (Cost + Mpr) / DTS)),
	seller_writeln(['-Seller: ROUND 3: PRICE BAD: ', AskingPrice, '.']).


/* NTH ROUND */

/* We try to make an offer they'll like that will yield more money for us. */
/* First, if they don't budge and their offer gives us some profit, take it. */
/* Second, if they don't budge and their offer doesn't give us minprof, make a new one. */
/* Third, if they're still budging and their offer gives us respectable profit, take it. */
/* Fourth, if they're still budging and their offer doesn't give us resprof, make a new one. */

seller(Cost, DTS, [BuyAttempt, SellAttempt, BA2, SA2 | Tail], AskingPrice) :-
	'seller:param'(minprof, Mp),
	BuyAttempt =:= BA2,
	BA2 > (Cost + (DTS * Mp)),
	AskingPrice is (0 - BA2),
	seller_writeln(['-Seller: ROUND N: DETECTED STATIC: GOOD: ', AskingPrice, '.']).

seller(Cost, DTS, [BuyAttempt, SellAttempt, BA2, SA2 | Tail], AskingPrice) :-
	'seller:param'(minprof, Mp),
	BuyAttempt =:= BA2,
	BA2 < (Cost + (DTS * Mp)),
	AskingPrice is (Cost + (DTS * Mp)),
	seller_writeln(['-Seller: ROUND N: DETECTED STATIC: BAD: ', AskingPrice, '.']).

seller(Cost, DTS, [BuyAttempt, SellAttempt, BA2, SA2 | Tail], AskingPrice) :-
	'seller:param'(initmod, Imod),
	'seller:param'(minprof, Mpr),
	BuyAttempt >= (Cost + (Mpr * DTS)),
	AskingPrice is BuyAttempt,
	seller_writeln(['-Seller: ROUND N: ACCEPTABLE: ', AskingPrice, '.']).

seller(Cost, DTS, [BuyAttempt, SellAttempt, BA2, SA2 | Tail], AskingPrice) :-
	'seller:param'(initmod, Imod),
	'seller:param'(minprof, Mpr),
	BuyAttempt < (Cost + (Mpr * DTS)),
	AskingPrice is (Cost + (Mpr * DTS)),
	seller_writeln(['-Seller: ROUND N: STILL BAD: ', AskingPrice, '.']).


/* SELLER WRITE FUNCTIONS */

seller_writeln(_) :-
	mediator_parameter(no_explanation),
	!.

seller_writeln([]) :-
	nl,
	!.

seller_writeln([H|T]) :-
	write(H),
	seller_writeln(T),
	!.